function editClasspath
% editClasspath - Add MatlabServer.jar to classpath.txt file. 
% 
% SUMMARY
%     Modelit provides a number of functions that require the static path
%     to be set. The static path is set at startup and read from the file
%     javaclasspath.txt. editClasspath writes this file and should be run
%     whenever the software is installed in a new location.
% 
% CALL
%     editClasspath
% 
% INPUT
%     no input required
% 
% OUTPUT
%     terminateApplication: 
%               if true: terminate application (compiled mode) or terminate
%               Matlab session (compiled mode)

%Modelit 2005-2021

jarFile=which('MatlabServer.jar'); %add this to static path
if isempty(jarFile)
    warndlg('File "MatlabServer.jar" not found on path. Look up and add directory to path first.','Edit classpath failed')
    return
end

pth=fileparts(which('startup.m'));
if isempty(pth)
    pth=pwd;
end

%WIJZ ZIJPP 20150228: for Matlab version 2014b and up: use
%javaclasspath.txt
WRITECLASSPATH=false; %==> withe javaclassptah.txt
try
    if verLessThan('matlab','8')
        %version 2012a or earlier ==> write classpath.txt
        WRITECLASSPATH=true;
    end
catch
    %even older version (verLessThan not found) ==> write classpath.txt
    WRITECLASSPATH=true;
end

if WRITECLASSPATH
    cpTarget=[pth,filesep,'classpath.txt']; %write classpath.txt here    
else
    cpTarget=[pth,filesep,'javaclasspath.txt']; %write javaclasspath.txt here
end

if exist(cpTarget,'file')
    warndlg(sprintf('File %s already exists. To avoid any unintential changes tou your Javaclasspath settings, editClasspath will not edit this file.',cpTarget),'Edit classpath failed')
    return
end


if WRITECLASSPATH
    %Read initial lines from classpath.txt in toolbox/local
    %the class path is not OK
    cpSource=fullfile(matlabroot,'toolbox','local','classpath.txt');
    if ~exist(cpSource,'file')
        warndlg('Modelit jarfile installer: The file classpath.txt could not be located','Edit classpath failed')
        return
    end
    
    %read the current classPath
    cp_txt=readcell_util(cpSource); %ZIJPP 20200603: readcell is generic function, avoid confusion
    
    cp_txt=writeClassPath(cp_txt,cpSource,jarFile);
    writestr(cpTarget,cp_txt);
else
    %Entire process is much more simple: just insert <before> and then list
    %custom jarfiles. in this case there is only 1 custom jarFile
    %(matlabserver.jar)
    cp_txt={'<before>',jarFile}';
    writestr(cpTarget,cp_txt);
end

msgbox(char('The java class path has now been set',...
        'Restart the Matlab session to apply the new classpath'),...
        'Set JAVA path at first time use');
%____________________________________________________________________ 
function cp_txt=writeClassPath(cp_txt,cpSource,jarFile)
%find last line that contains word: "$matlabroot" and insert line after
%this line
LASTSPEC=[];
for k=length(cp_txt):-1:1
    if ~isempty(findstr('$matlabroot',cp_txt{k}))
        LASTSPEC=k;
        break
    end
end
if isempty(LASTSPEC)
    error('Modelit jarfile installer: %s is not a valid classpath file',cpSource);
end

% Typical line for insertion:
% c:\modelit\java\mdltTable.jar

cp_txt=cat(1,...
    cp_txt(1:LASTSPEC),...
    {jarFile},...
    cp_txt(LASTSPEC+1:end));
%__________________________________________________________________________
function strs = readcell_util(fname, n)
% readcell_util - lees character array weg van file
% 
% CALL:
%  strs = readcell_util(fname, n)
% 
% INPUT:
%  fname: in te lezen file
%  n:     aantal te lezen regels 
% 
% OUTPUT:
%  str:
%   
% See also: writestr, readstr

if nargin<2
    n=inf;
end
strs={};
fid = fopen(fname,'r');
if fid==-1
    error('File %s could not be opened for reading',fname);
end
r=1;
while r<=n
    tline = fgetl(fid);
    if ~ischar(tline), break, end
    strs{end+1,1}=tline;
    r=r+1;
end
fclose(fid);
%__________________________________________________________________________
function writestr(fname,str)
% writestr - schrijf character array weg naar file
% 
% CALL
%   writestr(fname,str)
% 
% INPUT
%   fname : weg te schrijven file
%   str   : weg te schrijven char array
% 
% OUTPUT
%   none
%   
% EXAMPLE
%     hwin=ticp
%     hwait=waitbar(0,'Generate report');
%     strs={};
% 
%     for k=1:N
%         waitbar((k-.5)/N,hwait);
%         strs{end+1}=makeReportLine(k,....);
%     end
%     writestr(fname,strs(:)); <<<Note! vertical concatination is needed to
%                                 create multiline report
%     close(hwait);
%     tocp
% 
% SEE ALSO: readstr
fid = fopen(fname,'w');
if fid==-1
    %Maybe the directory does not exist
    [dummy,curfname,ext]=fileparts(fname);
    curfname=[curfname,ext];
    fid = fopen(curfname,'w');
    if fid==-1
        error('File %s nor %s could be opened for writing',fname,curfname);
    end
    dprintf('File %s could not be opened, save %s instead',fname,curfname);
end
if iscell(str)
    %str is MxN cell array. Iedere cell moet char array bevatten
    %1. voeg rijen in hor richting samen
    %2. print rij
    
    for k=1:size(str,1)
        fprintf(fid,'%s\r\n',cat(2,str{k,:}));
    end
else
    for k=1:size(str,1)
        fprintf(fid,'%s\r\n',[deblank(str(k,:)),'   ' ]);
    end
end
fclose(fid);

